@@ -63,8 +63,8 @@ class MemoryCache(Cache):
                 event_to_queue['pending_delete'] = False
             else:
                 self.logger.warning(
-                    "Could not requeue event with id {}. "
-                    "It does not appear to be in the cache.".format(event['id']))
+                    "Could not requeue event with id %s. It does not appear to be in the cache.",
+                    event['id'])
 
     # ----------------------------------------------------------------------
     def delete_queued_events(self):
@@ -91,5 +91,5 @@ class MemoryCache(Cache):
             event = self._cache.pop(event_id, None)
             if not event:
                 self.logger.warning(
-                    "Could not delete event with id {}. "
-                    "It does not appear to be in the cache.".format(event_id))
+                    "Could not delete event with id %s. It does not appear to be in the cache.",
+                    event_id)
