@@ -5,7 +5,7 @@
 
 from datetime import datetime
 from logging import getLogger as get_logger
-from queue import Empty, Queue
+from queue import Empty, PriorityQueue
 from socket import gaierror as socket_gaierror
 from threading import Event, Thread
 
@@ -14,7 +14,7 @@ from limits.storage import MemoryStorage
 from limits.strategies import FixedWindowRateLimiter
 
 from logstash_async.constants import constants
-from logstash_async.database import DatabaseCache, DatabaseLockedError
+from logstash_async.database import DatabaseCache, DatabaseDiskIOError, DatabaseLockedError
 from logstash_async.memory_cache import MemoryCache
 from logstash_async.utils import safe_log_via_print
 
@@ -46,7 +46,7 @@ class LogProcessingWorker(Thread):  # pylint: disable=too-many-instance-attribut
 
         self._shutdown_event = Event()
         self._flush_event = Event()
-        self._queue = Queue()
+        self._queue = PriorityQueue()
 
         self._event = None
         self._database = None
@@ -128,7 +128,7 @@ class LogProcessingWorker(Thread):  # pylint: disable=too-many-instance-attribut
                 self._flush_queued_events(force=force_flush)
                 self._delay_processing()
                 self._expire_events()
-            except (DatabaseLockedError, ProcessingError):
+            except (DatabaseLockedError, ProcessingError, DatabaseDiskIOError):
                 if self._shutdown_requested():
                     return
 
@@ -145,8 +145,15 @@ class LogProcessingWorker(Thread):  # pylint: disable=too-many-instance-attribut
             self._write_event_to_database()
         except DatabaseLockedError as exc:
             self._safe_log(
-                u'debug',
-                u'Database is locked, will try again later (queue length %d)',
+                'debug',
+                'Database is locked, will try again later (queue length %d)',
+                self._queue.qsize(),
+                exc=exc)
+            raise
+        except DatabaseDiskIOError as exc:
+            self._safe_log(
+                'debug',
+                'Disk I/O error, will try again later (queue length %d)',
                 self._queue.qsize(),
                 exc=exc)
             raise
@@ -160,7 +167,7 @@ class LogProcessingWorker(Thread):  # pylint: disable=too-many-instance-attribut
     def _expire_events(self):
         try:
             self._database.expire_events()
-        except DatabaseLockedError:
+        except (DatabaseLockedError, DatabaseDiskIOError):
             # Nothing to handle, if it fails, we will either successfully publish
             # these messages next time or we will delete them on the next pass.
             pass
@@ -168,8 +175,8 @@ class LogProcessingWorker(Thread):  # pylint: disable=too-many-instance-attribut
     # ----------------------------------------------------------------------
     def _log_processing_error(self, exception):
         self._safe_log(
-            u'exception',
-            u'Log processing error (queue size: %3s): %s',
+            'exception',
+            'Log processing error (queue size: %3s): %s',
             self._queue.qsize(),
             exception,
             exc=exception)
@@ -212,18 +219,18 @@ class LogProcessingWorker(Thread):  # pylint: disable=too-many-instance-attribut
             try:
                 events = [event['event_text'] for event in queued_events]
                 self._send_events(events)
-            # exception types for which we do not want a stack trace
+            # Log connection and network errors as warnings as they are rather harmless
             except (ConnectionError, TimeoutError, socket_gaierror) as exc:
                 self._safe_log(
-                    u'error',
-                    u'An error occurred while sending events: %s',
+                    'warning',
+                    'An error occurred while sending events: %s',
                     exc)
                 self._database.requeue_queued_events(queued_events)
                 break
             except Exception as exc:
                 self._safe_log(
-                    u'exception',
-                    u'An error occurred while sending events: %s',
+                    'exception',
+                    'An error occurred while sending events: %s',
                     exc,
                     exc=exc)
                 self._database.requeue_queued_events(queued_events)
@@ -238,13 +245,20 @@ class LogProcessingWorker(Thread):  # pylint: disable=too-many-instance-attribut
             return self._database.get_queued_events()
         except DatabaseLockedError as exc:
             self._safe_log(
-                u'debug',
-                u'Database is locked, will try again later (queue length %d)',
+                'debug',
+                'Database is locked, will try again later (queue length %d)',
                 self._queue.qsize(),
                 exc=exc)
+        except DatabaseDiskIOError as exc:
+            self._safe_log(
+                'debug',
+                'Disk I/O error, will try again later (queue length %d)',
+                self._queue.qsize(),
+                exc=exc)
+            raise
         except Exception as exc:
             # just log the exception and hope we can recover from the error
-            self._safe_log(u'exception', u'Error retrieving queued events: %s', exc, exc=exc)
+            self._safe_log('exception', 'Error retrieving queued events: %s', exc, exc=exc)
 
         return None
 
@@ -252,7 +266,7 @@ class LogProcessingWorker(Thread):  # pylint: disable=too-many-instance-attribut
     def _delete_queued_events_from_database(self):
         try:
             self._database.delete_queued_events()
-        except DatabaseLockedError:
+        except (DatabaseLockedError, DatabaseDiskIOError):
             pass  # nothing to handle, if it fails, we delete those events in a later run
 
     # ----------------------------------------------------------------------
@@ -271,7 +285,7 @@ class LogProcessingWorker(Thread):  # pylint: disable=too-many-instance-attribut
 
     # ----------------------------------------------------------------------
     def _log_general_error(self, exc):
-        self._safe_log(u'exception', u'An unexpected error occurred: %s', exc, exc=exc)
+        self._safe_log('exception', 'An unexpected error occurred: %s', exc, exc=exc)
 
     # ----------------------------------------------------------------------
     def _safe_log(self, log_level, message, *args, **kwargs):
@@ -285,8 +299,7 @@ class LogProcessingWorker(Thread):  # pylint: disable=too-many-instance-attribut
             if rate_limit_allowed == 1:
                 # extend the message to indicate future rate limiting
                 message = \
-                    u'{} (rate limiting effective, ' \
-                    'further equal messages will be limited)'.format(message)
+                    f'{message} (rate limiting effective, further equal messages will be limited'
 
             self._safe_log_impl(log_level, message, *args, **kwargs)
 
@@ -324,6 +337,6 @@ class LogProcessingWorker(Thread):  # pylint: disable=too-many-instance-attribut
         if queue_size:
             self._safe_log(
                 'warn',
-                u'Non-empty queue while shutting down ({} events pending). '
-                u'This indicates a previous error.'.format(queue_size),
+                f'Non-empty queue while shutting down ({queue_size} events pending). '
+                'This indicates a previous error.',
                 extra=dict(queue_size=queue_size))
